# frozen_string_literal: true

require 'influxdb-client'

module Latias
  # Influxdb
  module Influxdb
    # Bucket
    class Bucket
      attr_reader :url, :token, :bucket, :org, :use_ssl, :verify_mode, :precision

      def initialize
        @query = ''
      end

      def call
        all_query = "from(bucket: \"#{use_value('bucket')}\") #{@query}"
        @query = ''
        @client.create_query_api.query(query: all_query, org: use_value('org'))
      end

      def yield(query)
        @query += "#{new_command} yield(#{query})"
        self
      end

      def range(query)
        @query += "#{new_command} range(#{query})"
        self
      end

      def filter(query)
        @query += "#{new_command} filter(#{query})"
        self
      end

      def real_query(query)
        @query += query
      end

      def client_connection
        @client = InfluxDB2::Client.new(use_value('url'), use_value('token'),
                                        use_ssl: use_value('use_ssl'),
                                        verify_mode: use_value('verify_mode'),
                                        precision: InfluxDB2::WritePrecision::NANOSECOND)
        self
      end

      def client_reconnection
        client_connection
      end

      private

      def use_value(key)
        b = get_binding("self")
        b.eval("@#{key}") || Latias::Influxdb.configuration.send(key)
      end

      def new_command
        '|> '
      end

      def get_binding(param)
        binding
      end
    end
  end
end
